package java.lang;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.StringTokenizer;

public class Runtime {
   private static Runtime currentRuntime = new Runtime();

   public static Runtime getRuntime() {
      return currentRuntime;
   }

   private Runtime() {
   }

   public void exit(int var1) {
      SecurityManager var2 = System.getSecurityManager();
      if (var2 != null) {
         var2.checkExit(var1);
      }

      Shutdown.exit(var1);
   }

   public void addShutdownHook(Thread var1) {
      SecurityManager var2 = System.getSecurityManager();
      if (var2 != null) {
         var2.checkPermission(new RuntimePermission("shutdownHooks"));
      }

      Shutdown.add(var1);
   }

   public boolean removeShutdownHook(Thread var1) {
      SecurityManager var2 = System.getSecurityManager();
      if (var2 != null) {
         var2.checkPermission(new RuntimePermission("shutdownHooks"));
      }

      return Shutdown.remove(var1);
   }

   public void halt(int var1) {
      SecurityManager var2 = System.getSecurityManager();
      if (var2 != null) {
         var2.checkExit(var1);
      }

      Shutdown.halt(var1);
   }

   public static void runFinalizersOnExit(boolean var0) {
      SecurityManager var1 = System.getSecurityManager();
      if (var1 != null) {
         try {
            var1.checkExit(0);
         } catch (SecurityException var3) {
            throw new SecurityException("runFinalizersOnExit");
         }
      }

      Shutdown.setRunFinalizersOnExit(var0);
   }

   private native Process execInternal(String[] var1, String[] var2, String var3) throws IOException;

   public Process exec(String var1) throws IOException {
      return this.exec((String)var1, (String[])null);
   }

   public Process exec(String var1, String[] var2) throws IOException {
      return this.exec((String)var1, var2, (File)null);
   }

   public Process exec(String var1, String[] var2, File var3) throws IOException {
      int var4 = 0;
      StringTokenizer var6 = new StringTokenizer(var1);
      var4 = var6.countTokens();
      String[] var5 = new String[var4];
      var6 = new StringTokenizer(var1);

      for(int var8 = 0; var6.hasMoreTokens(); var5[var8++] = var6.nextToken()) {
      }

      return this.exec(var5, var2, var3);
   }

   public Process exec(String[] var1) throws IOException {
      return this.exec((String[])var1, (String[])null);
   }

   public Process exec(String[] var1, String[] var2) throws IOException {
      return this.exec((String[])var1, var2, (File)null);
   }

   public Process exec(String[] var1, String[] var2, File var3) throws IOException {
      var1 = (String[])var1.clone();
      var2 = var2 != null ? (String[])var2.clone() : null;
      if (var1.length == 0) {
         throw new IndexOutOfBoundsException();
      } else {
         for(int var4 = 0; var4 < var1.length; ++var4) {
            if (var1[var4] == null) {
               throw new NullPointerException();
            }
         }

         if (var2 != null) {
            for(int var5 = 0; var5 < var2.length; ++var5) {
               if (var2[var5] == null) {
                  throw new NullPointerException();
               }
            }
         }

         SecurityManager var9 = System.getSecurityManager();
         if (var9 != null) {
            var9.checkExec(var1[0]);
         }

         String var6 = var3 == null ? null : var3.getPath();
         return this.execInternal(var1, var2, var6);
      }
   }

   public native long freeMemory();

   public native long totalMemory();

   public native void gc();

   private static native void runFinalization0();

   public void runFinalization() {
      runFinalization0();
   }

   public native void traceInstructions(boolean var1);

   public native void traceMethodCalls(boolean var1);

   public void load(String var1) {
      this.load0(System.getCallerClass(), var1);
   }

   synchronized void load0(Class var1, String var2) {
      SecurityManager var3 = System.getSecurityManager();
      if (var3 != null) {
         var3.checkLink(var2);
      }

      if (!(new File(var2)).isAbsolute()) {
         throw new UnsatisfiedLinkError("Expecting an absolute path of the library: " + var2);
      } else {
         ClassLoader.loadLibrary(var1, var2, true);
      }
   }

   public void loadLibrary(String var1) {
      this.loadLibrary0(System.getCallerClass(), var1);
   }

   synchronized void loadLibrary0(Class var1, String var2) {
      SecurityManager var3 = System.getSecurityManager();
      if (var3 != null) {
         var3.checkLink(var2);
      }

      if (var2.indexOf(File.separatorChar) != -1) {
         throw new UnsatisfiedLinkError("Directory separator should not appear in library name: " + var2);
      } else {
         ClassLoader.loadLibrary(var1, var2, false);
      }
   }

   public InputStream getLocalizedInputStream(InputStream var1) {
      return var1;
   }

   public OutputStream getLocalizedOutputStream(OutputStream var1) {
      return var1;
   }
}
